@@ -242,10 +242,12 @@ pyg_type_name (PyObject *self, PyObject *args)
     GType type;
     const gchar *name;
 
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_name is deprecated; "
 		   "use GType.name instead"))
         return NULL;
+#endif
     
     if (!PyArg_ParseTuple(args, "O:gobject.type_name", &gtype))
 	return NULL;
@@ -263,18 +265,20 @@ pyg_type_from_name (PyObject *self, PyObject *args)
 {
     const gchar *name;
     GType type;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_from_name is deprecated; "
 		   "use GType.from_name instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "s:gobject.type_from_name", &name))
 	return NULL;
     type = g_type_from_name(name);
     if (type != 0)
 	return pyg_type_wrapper_new(type);
-    PyErr_SetString(PyExc_RuntimeError, "unknown type name");
+    PyErr_Format(PyExc_RuntimeError, "%s: unknown type name: %s",
+		 PyString_AsString(PyObject_Repr((PyObject*)self)),
+		 name);
     return NULL;
 }
 
@@ -283,12 +287,12 @@ pyg_type_parent (PyObject *self, PyObject *args)
 {
     PyObject *gtype;
     GType type, parent;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_parent is deprecated; "
 		   "use GType.parent instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "O:gobject.type_parent", &gtype))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -305,12 +309,12 @@ pyg_type_is_a (PyObject *self, PyObject *args)
 {
     PyObject *gtype, *gparent;
     GType type, parent;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_is_a is deprecated; "
 		   "use GType.is_a instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "OO:gobject.type_is_a", &gtype, &gparent))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -326,12 +330,12 @@ pyg_type_children (PyObject *self, PyObject *args)
     PyObject *gtype, *list;
     GType type, *children;
     guint n_children, i;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_children is deprecated; "
 		   "use GType.children instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "O:gobject.type_children", &gtype))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -357,12 +361,12 @@ pyg_type_interfaces (PyObject *self, PyObject *args)
     PyObject *gtype, *list;
     GType type, *interfaces;
     guint n_interfaces, i;
-
+#if 0
     if (PyErr_Warn(PyExc_DeprecationWarning,
 		   "gobject.type_interfaces is deprecated; "
 		   "use GType.interfaces instead"))
         return NULL;
-    
+#endif    
     if (!PyArg_ParseTuple(args, "O:gobject.type_interfaces", &gtype))
 	return NULL;
     if ((type = pyg_type_from_object(gtype)) == 0)
@@ -925,7 +929,17 @@ add_properties (GType instance_type, PyObject *properties)
 	if (pspec) {
 	    g_object_class_install_property(oclass, 1, pspec);
 	} else {
+            PyObject *type, *value, *traceback;
 	    ret = FALSE;
+            PyErr_Fetch(&type, &value, &traceback);
+            if (PyString_Check(value)) {
+                char msg[256];
+                g_snprintf(msg, 256, "%s (while registering property '%s' for GType '%s')",
+                           PyString_AsString(value), prop_name, g_type_name(instance_type));
+                Py_DECREF(value);
+                value = PyString_FromString(msg);
+            }
+            PyErr_Restore(type, value, traceback);
 	    break;
 	}
     }
@@ -975,7 +989,7 @@ _wrap_pyg_type_register(PyObject *self, PyObject *args)
 {
     PyTypeObject *class;
 
-    if (!PyArg_ParseTuple(args, "O:gobject.type_register", &class))
+    if (!PyArg_ParseTuple(args, "O!:gobject.type_register", &PyType_Type, &class))
 	return NULL;
     if (!PyType_IsSubtype(class, &PyGObject_Type)) {
 	PyErr_SetString(PyExc_TypeError,"argument must be a GObject subclass");
@@ -988,20 +1002,8 @@ _wrap_pyg_type_register(PyObject *self, PyObject *args)
     {
         if (pyg_type_register(class, NULL))
             return NULL;
-    } else {
-        gchar *msg;
-          /* already registered */
-        msg = g_strdup_printf("Class %s is already GObject-registered; "
-			      "Please note that classes containing any of the "
-			      "attributes __gtype_name__, __gproperties__, or "
-			      "__gsignals__ are now automatically registered.",
-			      class->tp_name);
-        if (PyErr_Warn(PyExc_DeprecationWarning, msg)) {
-            g_free(msg);
-            return NULL;
-        }
-        g_free(msg);
     }
+ 
     Py_INCREF(class);
     return (PyObject *) class;
 }
@@ -1099,7 +1101,7 @@ pygobject__g_instance_init(GTypeInstance   *instance,
 }
 
 int
-pyg_type_register(PyTypeObject *class, char *type_name)
+pyg_type_register(PyTypeObject *class, const char *type_name)
 {
     PyObject *gtype, *gsignals, *gproperties, *overridden_signals;
     GType parent_type, instance_type;
@@ -1121,7 +1123,7 @@ pyg_type_register(PyTypeObject *class, char *type_name)
 	0,    /* n_preallocs */
 	(GInstanceInitFunc) pygobject__g_instance_init
     };
-
+    gchar *new_type_name;
 
     /* find the GType of the parent */
     parent_type = pyg_type_from_object((PyObject *)class);
@@ -1129,8 +1131,10 @@ pyg_type_register(PyTypeObject *class, char *type_name)
 	return -1;
     }
 
-    if (!type_name)
-	type_name = get_type_name_for_class(class);
+    if (type_name)
+        new_type_name = (gchar *) type_name; /* care is taken below not to free this */
+    else
+	new_type_name = get_type_name_for_class(class);
     
     /* set class_data that will be passed to the class_init function. */
     type_info.class_data = class;
@@ -1141,9 +1145,10 @@ pyg_type_register(PyTypeObject *class, char *type_name)
     type_info.instance_size = query.instance_size;
 
     /* create new typecode */
-    instance_type = g_type_register_static(parent_type, type_name,
+    instance_type = g_type_register_static(parent_type, new_type_name,
 					   &type_info, 0);
-    g_free(type_name);
+    if (type_name == NULL)
+        g_free(new_type_name);
     if (instance_type == 0) {
 	PyErr_SetString(PyExc_RuntimeError, "could not create new GType");
 	return -1;
@@ -1312,11 +1317,12 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
 {
     static char *kwlist[] = { "type", NULL };
     PyObject *py_itype, *list;
-    GObjectClass *class;
+    GObjectClass *class = NULL;
     GType itype;
     guint n;
     guint *ids;
     guint i;
+    gpointer iface = NULL;
 
     if (!PyArg_ParseTupleAndKeywords(args, kwargs,
                                      "O:gobject.signal_list_names",
@@ -1325,31 +1331,36 @@ pyg_signal_list_names (PyObject *self, PyObject *args, PyObject *kwargs)
     if ((itype = pyg_type_from_object(py_itype)) == 0)
 	return NULL;
 
-    if (!G_TYPE_IS_INSTANTIATABLE(itype) && !G_TYPE_IS_INTERFACE(itype)) {
+    if (G_TYPE_IS_INSTANTIATABLE(itype)) {
+	class = g_type_class_ref(itype);
+	if (!class) {
+	    PyErr_SetString(PyExc_RuntimeError,
+			    "could not get a reference to type class");
+	    return NULL;
+	}
+    } else if (!G_TYPE_IS_INTERFACE(itype)) {
 	PyErr_SetString(PyExc_TypeError,
 			"type must be instantiable or an interface");
 	return NULL;
+    } else {
+        iface = g_type_default_interface_ref(itype);
     }
-
-    class = g_type_class_ref(itype);
-    if (!class) {
-	PyErr_SetString(PyExc_RuntimeError,
-			"could not get a reference to type class");
-	return NULL;
-    }
+    
     ids = g_signal_list_ids(itype, &n);
 
     list = PyTuple_New((gint)n);
-    if (list == NULL) {
-	g_free(ids);
-	g_type_class_unref(class);
-	return NULL;
+    if (list != NULL) {
+	for (i = 0; i < n; i++)
+	    PyTuple_SetItem(list, i,
+			    PyString_FromString(g_signal_name(ids[i])));
     }
-
-    for (i = 0; i < n; i++)
-	PyTuple_SetItem(list, i, PyString_FromString(g_signal_name(ids[i])));
+    
     g_free(ids);
-    g_type_class_unref(class);
+    if (class)
+	   g_type_class_unref(class);
+    else
+       g_type_default_interface_unref(iface);
+
     return list;
 }
 
@@ -1649,12 +1660,12 @@ pyg_object_new (PyGObject *self, PyObject *args, PyObject *kwargs)
     g_free(params);
     g_type_class_unref(class);
     
-    if (obj)
+    if (obj) {
 	self = (PyGObject *) pygobject_new_full((GObject *)obj, FALSE);
-    else
+        g_object_unref(obj);
+        pygobject_sink(obj);
+    } else
         self = NULL;
-    g_object_unref(obj);
-    pygobject_sink(obj);
 
     return (PyObject *) self;
 }
@@ -2596,6 +2607,43 @@ pyg_set_object_has_new_constructor(GType type)
     g_type_set_qdata(type, pygobject_has_updated_constructor_key, GINT_TO_POINTER(1));
 }
 
+#define GET_INT(x) (((PyIntObject*)x)->ob_ival)
+PyObject *
+pyg_integer_richcompare(PyObject *v, PyObject *w, int op)
+{
+    PyObject *result;
+    gboolean t;
+
+    switch (op) {
+    case Py_EQ: t = GET_INT(v) == GET_INT(w); break;
+    case Py_NE: t = GET_INT(v) != GET_INT(w); break;
+    case Py_LE: t = GET_INT(v) <= GET_INT(w); break;
+    case Py_GE: t = GET_INT(v) >= GET_INT(w); break;
+    case Py_LT: t = GET_INT(v) <  GET_INT(w); break;
+    case Py_GT: t = GET_INT(v) >  GET_INT(w); break;
+    default: g_assert_not_reached();
+    }
+
+    result = t ? Py_True : Py_False;
+    Py_INCREF(result);
+    return result;
+}
+#undef GET_INT
+
+static void
+_log_func(const gchar *log_domain,
+          GLogLevelFlags log_level,
+          const gchar *message,
+          gpointer user_data)
+{
+    PyGILState_STATE state;
+    PyObject* warning = user_data;
+
+    state = pyg_gil_state_ensure();
+    PyErr_Warn(warning, (char *) message);
+    pyg_gil_state_release(state);
+}
+
 /* ----------------- gobject module initialisation -------------- */
 
 struct _PyGObject_Functions pygobject_api_functions = {
@@ -2687,6 +2735,8 @@ initgobject(void)
 {
     PyObject *m, *d, *o, *tuple;
     PyObject *descr;
+    PyObject *warning;
+    
     PyGParamSpec_Type.ob_type = &PyType_Type;
     
     m = Py_InitModule("gobject", pygobject_functions);
@@ -2865,4 +2915,14 @@ initgobject(void)
     pyg_register_gtype_custom(G_TYPE_STRV,
 			      _pyg_strv_from_gvalue,
 			      _pyg_strv_to_gvalue);
+
+    warning = PyErr_NewException("gobject.Warning", PyExc_Warning, NULL);
+    PyDict_SetItemString(d, "Warning", warning);
+    g_log_set_handler("GLib", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _log_func, warning);
+    g_log_set_handler("GLib-GObject", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _log_func, warning);
+    g_log_set_handler("GThread", G_LOG_LEVEL_CRITICAL|G_LOG_LEVEL_WARNING,
+                      _log_func, warning);
+
 }
