@@ -28,20 +28,21 @@
 #include "pygobject-private.h"
 
 #define GET_INT_VALUE(x) (((PyIntObject*)x)->ob_ival)
-static int
-pyg_flags_compare(PyGFlags *self, PyObject *other)
+
+static PyObject *
+pyg_flags_richcompare(PyGFlags *self, PyObject *other, int op)
 {
-    if (!PyInt_CheckExact(other) && ((PyGFlags*)other)->gtype != self->gtype) {
-	PyErr_Warn(PyExc_Warning, "comparing different flags types");
-	return -1;
+    if (!PyInt_Check(other)) {
+        Py_INCREF(Py_NotImplemented);
+        return Py_NotImplemented;
     }
 
-    if (GET_INT_VALUE(self) == GET_INT_VALUE(other))
-      return 0;
-    else if (GET_INT_VALUE(self) < GET_INT_VALUE(other))
-      return -1;
-    else
-      return 1;
+    if (PyObject_TypeCheck(other, &PyGFlags_Type) && ((PyGFlags*)other)->gtype != self->gtype) {
+	PyErr_Warn(PyExc_Warning, "comparing different flags types");
+	return NULL;
+    }
+
+    return pyg_integer_richcompare((PyObject *)self, other, op);
 }
 
 static char *
@@ -213,14 +214,15 @@ pyg_flags_add (PyObject *   module,
     ((PyTypeObject *)stub)->tp_flags &= ~Py_TPFLAGS_BASETYPE;
     ((PyTypeObject *)stub)->tp_new = pyg_flags_new;
 
-    PyDict_SetItemString(((PyTypeObject *)stub)->tp_dict,
-			 "__module__",
-			 PyString_FromString(PyModule_GetName(module)));
+    if (module) {
+        PyDict_SetItemString(((PyTypeObject *)stub)->tp_dict,
+                             "__module__",
+                             PyString_FromString(PyModule_GetName(module)));
     
-    /* Add it to the module name space */
-    PyModule_AddObject(module, (char*)typename, stub);
-    Py_INCREF(stub);
-
+          /* Add it to the module name space */
+        PyModule_AddObject(module, (char*)typename, stub);
+        Py_INCREF(stub);
+    }
     g_type_set_qdata(gtype, pygflags_class_key, stub);
 
     o = pyg_type_wrapper_new(gtype);
@@ -240,11 +242,13 @@ pyg_flags_add (PyObject *   module,
             
       PyDict_SetItem(values, PyInt_FromLong(eclass->values[i].value), item);
 
-      PyModule_AddObject(module,
-			 pyg_constant_strip_prefix(eclass->values[i].value_name,
-						   strip_prefix),
-			 item);
-      Py_INCREF(item);
+      if (module) {
+          PyModule_AddObject(module,
+                             pyg_constant_strip_prefix(eclass->values[i].value_name,
+                                                       strip_prefix),
+                             item);
+          Py_INCREF(item);
+      }
     }
     
     PyDict_SetItemString(((PyTypeObject *)stub)->tp_dict,
@@ -434,7 +438,7 @@ PyTypeObject PyGFlags_Type = {
 	0,                			  /* tp_print */
 	0,					  /* tp_getattr */
 	0,					  /* tp_setattr */
-	(cmpfunc)pyg_flags_compare,		  /* tp_compare */
+	0,					  /* tp_compare */
 	(reprfunc)pyg_flags_repr,		  /* tp_repr */
 	&pyg_flags_as_number,			  /* tp_as_number */
 	0,					  /* tp_as_sequence */
@@ -449,7 +453,7 @@ PyTypeObject PyGFlags_Type = {
 	0,      				  /* tp_doc */
 	0,					  /* tp_traverse */
 	0,					  /* tp_clear */
-	0,					  /* tp_richcompare */
+	(richcmpfunc)pyg_flags_richcompare,	  /* tp_richcompare */
 	0,					  /* tp_weaklistoffset */
 	0,					  /* tp_iter */
 	0,					  /* tp_iternext */
