@@ -66,6 +66,7 @@ _pygtk_style_helper_new(GtkStyle *style, int type, gpointer array)
 						   &PyGtkStyleHelper_Type);
     if (self == NULL)
 	return NULL;
+
     self->style = g_object_ref(style);
     self->type = type;
     self->array = array;
@@ -75,9 +76,7 @@ _pygtk_style_helper_new(GtkStyle *style, int type, gpointer array)
 static void
 pygtk_style_helper_dealloc(PyGtkStyleHelper_Object *self)
 {
-    pyg_unblock_threads();
     g_object_unref(self->style);
-    pyg_block_threads();
     PyObject_DEL(self);
 }
 
@@ -149,9 +148,7 @@ pygtk_style_helper_setitem(PyGtkStyleHelper_Object *self, int pos,
 		return -1;
 	    }
 	    if (array[pos]) {
-		pyg_unblock_threads();
 		g_object_unref(array[pos]);
-		pyg_block_threads();
 	    }
 	    array[pos] = GDK_GC(g_object_ref(pygobject_get(value)));
 	    return 0;
@@ -166,9 +163,7 @@ pygtk_style_helper_setitem(PyGtkStyleHelper_Object *self, int pos,
 		return -1;
 	    }
 	    if (array[pos]) {
-		pyg_unblock_threads();
 		g_object_unref(array[pos]);
-		pyg_block_threads();
 	    }
 	    if (value != Py_None)
 		array[pos] = GDK_PIXMAP(g_object_ref(pygobject_get(value)));
@@ -738,14 +733,6 @@ pygdk_atom_dealloc(PyGdkAtom_Object *self)
     PyObject_DEL(self);
 }
 
-static int
-pygdk_atom_compare(PyGdkAtom_Object *self, PyGdkAtom_Object *v)
-{
-    if (self->atom == v->atom) return 0;
-    if (self->atom > v->atom) return -1;
-    return 1;
-}
-
 static long
 pygdk_atom_hash(PyGdkAtom_Object *self)
 {
@@ -771,6 +758,45 @@ pygdk_atom_str(PyGdkAtom_Object *self)
     return pygdk_atom_repr(self);
 }
 
+static PyObject *
+pygdk_atom_richcompare(PyGdkAtom_Object *self, PyGdkAtom_Object *v, int op)
+{
+    PyObject *result = Py_NotImplemented;
+ 
+    if (PyString_Check(v)) {
+        PyObject *str = pygdk_atom_str(self);
+        result =  PyObject_RichCompare(str, (PyObject *)v, op);
+        Py_DECREF(str);
+        return result;
+    }
+    if (PyGdkAtom_Check(v)) {
+        switch (op) {
+        case Py_LT:
+            result = (self->atom < v->atom) ? Py_True : Py_False;
+            break;
+        case Py_LE:
+            result = (self->atom <= v->atom) ? Py_True : Py_False;
+            break;
+        case Py_EQ:
+            result = (self->atom == v->atom) ? Py_True : Py_False;
+            break;
+        case Py_NE:
+            result = (self->atom != v->atom) ? Py_True : Py_False;
+            break;
+        case Py_GE:
+            result = (self->atom >= v->atom) ? Py_True : Py_False;
+            break;
+        case Py_GT:
+            result = (self->atom > v->atom) ? Py_True : Py_False;
+            break;
+        default:
+            break;
+        }
+    }
+    Py_INCREF(result);
+    return result;
+}
+
 PyTypeObject PyGdkAtom_Type = {
     PyObject_HEAD_INIT(NULL)
     0,
@@ -781,7 +807,7 @@ PyTypeObject PyGdkAtom_Type = {
     (printfunc)0,
     (getattrfunc)0,
     (setattrfunc)0,
-    (cmpfunc)pygdk_atom_compare,
+    (cmpfunc)0,
     (reprfunc)pygdk_atom_repr,
     0,
     0,
@@ -793,7 +819,10 @@ PyTypeObject PyGdkAtom_Type = {
     (setattrofunc)0,
     0,
     Py_TPFLAGS_DEFAULT,
-    NULL
+    NULL,
+    0,
+    0,
+    (richcmpfunc)pygdk_atom_richcompare,
 };
 
 typedef struct {
@@ -820,9 +849,7 @@ _pygtk_tree_model_row_new(GtkTreeModel *model, GtkTreeIter *iter)
 static void
 pygtk_tree_model_row_dealloc(PyGtkTreeModelRow *self)
 {
-    pyg_unblock_threads();
     g_object_unref(self->model);
-    pyg_block_threads();
     PyObject_DEL(self);
 }
 
@@ -1030,9 +1057,7 @@ _pygtk_tree_model_row_iter_new(GtkTreeModel *model, GtkTreeIter *parent_iter)
 static void
 pygtk_tree_model_row_iter_dealloc(PyGtkTreeModelRowIter *self)
 {
-    pyg_unblock_threads();
     g_object_unref(self->model);
-    pyg_block_threads();
     PyObject_DEL(self);
 }
 
